--[[
	Copyright (C) 2007 Nymbia

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
]]

PT3Bar = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
PT3Bar:RegisterDB("PT3BarDB")
local PT = AceLibrary("PeriodicTable-3.0")
local _G = getfenv(0)


-- These are for 2.0.x compatibility.  To be removed in 2.1, as itemids will work on the secure buttons
local function getname(id)
	return (GetItemInfo(id))
end
local function getid(name)
	if not name then
		return
	end
	local _,link = GetItemInfo(name)
	return link:match("item:(%d+)")
end
--

local buttons
local scale
local size = 36
local binds
local bindparent = CreateFrame("Frame",nil,UIParent)
function PT3Bar:InputSets(t,b,w)
	buttons = t
	scale = w
	binds = b
	self.InputSets = nil
end
local settables = {}
local locked = true
local options = {
	type = 'group',
	args = {
		lock = {
			type = 'toggle',
			name = "Lock",
			desc = "Lock",
			get = function()
				return locked
			end,
			set = function(v)
				locked = v
				if v then
					PT3Bar:UpdateAllButtons()
					for _,v in pairs(buttons) do
						v.moveoverlay:Hide()
					end
				else
					for _,v in pairs(buttons) do
						v:Show()
						v.moveoverlay:Show()
					end
				end
			end,
		},
		snap = {
			type = 'execute',
			name = "Snap",
			desc = "Snap",
			func = function()
				local x1, y1
				for _,frame in pairs(buttons) do
					local num = tonumber(frame:GetName():match("PT3Bar(%d+)"))
					if num == 1 then
						x1 = frame:GetLeft()
						y1 = frame:GetBottom()
					end
				end
				for _,frame in pairs(buttons) do
					local num = tonumber(frame:GetName():match("PT3Bar(%d+)"))
					if num ~= 1 then
						frame:ClearAllPoints()
						frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", x1+((size+4)*num)-(size+4), y1)
						PT3Bar.db.profile[num].x = frame:GetLeft()
						PT3Bar.db.profile[num].y = frame:GetBottom()
					end
				end
			end
		},
		tooltip = {
			type = 'toggle',
			name = "Tooltip",
			desc = "Tooltip",
			get = function()
				return PT3Bar.db.profile.tooltip
			end,
			set = function(v)
				PT3Bar.db.profile.tooltip = v
			end,
		},
	}
}
function PT3Bar:OnInitialize()
	self:RegisterChatCommand({"/pt3bar"}, options)
	self:RegisterDefaults('profile', {
		tooltip = true,
		['*'] = {}
	})
	
end

function PT3Bar:OnEnable(first)
	self:RegisterBucketEvent("BAG_UPDATE", 5)
	if first then
		local newbuttons = {}
		for k,v in ipairs(buttons) do
			local button = CreateFrame("CheckButton", "PT3Bar"..k, UIParent, "ActionBarButtonTemplate")
			button:SetMovable(true)
			button:SetClampedToScreen(true)
			
			button:SetScript("OnEnter", function()
				self:SetTooltip(button)
			end)
			button:SetScript("OnLeave", function()
				GameTooltip:Hide()
			end)
			button:SetScript("OnReceiveDrag", nil)
			button:SetScript("OnDragStart", nil)
			button:UnregisterAllEvents()
			button:SetAttribute("type", "item")
			button.icon = _G["PT3Bar"..k.."Icon"]
			button.icon:SetVertexColor(1,1,1,1)
			button.count = _G["PT3Bar"..k.."Count"]
			button.cooldown = _G["PT3Bar"..k.."Cooldown"]
			_G["PT3Bar"..k.."Name"]:Hide()
			_G["PT3Bar"..k.."HotKey"]:Hide()
			button:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", (self.db.profile[k].x or (200+(k*(size+4)))), (self.db.profile[k].y or 200))
			button:SetWidth(size)
			button:SetHeight(size)
			_G["PT3Bar"..k.."NormalTexture"]:SetWidth((size/36)*66)
			_G["PT3Bar"..k.."NormalTexture"]:SetHeight((size/36)*66)
			button:SetScale(scale)
			
			local moveoverlay = CreateFrame("Frame",nil,button)
			button.moveoverlay = moveoverlay

			moveoverlay:SetScript("OnMouseDown", function(frame, mousebtn)
				if not locked then
					if mousebtn == 'RightButton' then
						local v = not button.hidden
						button.hidden = v
						self.db.profile[k].hidden = v
						self:SetButtonHidden(button, v)
						
						moveoverlay:SetScript("OnMouseUp", nil)
					else
						button:StartMoving()
						moveoverlay:SetScript("OnMouseUp", function()
							button:StopMovingOrSizing()
							self.db.profile[k].x = button:GetLeft()
							self.db.profile[k].y = button:GetBottom()
						end)
					end
				end
			end)

			moveoverlay:EnableMouse(true)
			moveoverlay:SetFrameStrata("HIGH")
			moveoverlay:SetWidth(size)
			moveoverlay:SetHeight(size)
			moveoverlay:SetPoint("TOPLEFT",button,"TOPLEFT",0,0)
			moveoverlay:Hide()

			button.hidden = self.db.profile[k].hidden
			self:SetButtonHidden(button, button.hidden)
			
			newbuttons[v] = button
			settables[v] = PT(v)
		end
		buttons = newbuttons
	end
	ClearOverrideBindings(bindparent)
	for k,v in pairs(binds) do
		if v then
			SetOverrideBindingClick(bindparent, false, v, "PT3Bar"..k)
		end
	end
	for _,v in pairs(buttons) do
	        v:Show()
	end
	self:UpdateAllButtons()
	self:ScheduleRepeatingEvent(self.UpdateCooldowns, 5, self)
	
end

function PT3Bar:OnProfileEnable()
	for _, frame in pairs(buttons) do
		frame:ClearAllPoints()
		local k = tonumber(frame:GetName():match("PT3Bar(%d+)"))
		frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", (self.db.profile[k].x or (200+(k*(size+4)))), (self.db.profile[k].y or 200))
	end
end

function PT3Bar:OnDisable()
	ClearOverrideBindings(bindparent)
	for _,v in pairs(buttons) do
		v:Hide()
	end
end

function PT3Bar:BAG_UPDATE()
	if InCombatLockdown() then
		self:UpdateCountsAndAlphas()
		self:CancelAllCombatSchedules()
		self:ScheduleLeaveCombatAction(self.UpdateAllButtons, self)
	else
		self:UpdateAllButtons()
	end
end

function PT3Bar:UpdateCountsAndAlphas()
	for set,button in pairs(buttons) do
		if not button.item then
			return
		end
		local count = GetItemCount(button.item)
		if count == 0 then
			button:SetAlpha(0.2)
		end
		button.count:SetText(count)
	end
end

function PT3Bar:UpdateAllButtons()
	for set,button in pairs(buttons) do
		local _, firstv = next(settables[set])
		local currvalue = 0
		local curritem
		local currcount
		if type(firstv) == "table" then
			--multiset
			for _,t in ipairs(settables[set]) do
				for k,v in pairs(t) do
					v = tonumber(v)
					k = tonumber(k)
					if v and v > currvalue then
						local count = GetItemCount(k)
						if count > 0 then
							curritem = k
							currcount = count
							currvalue = v
						end
					end
				end
			end
		else
			--normal set
			for k,v in pairs(settables[set]) do
				v = tonumber(v)
				k = tonumber(k)
				if v and v > currvalue then
					local count = GetItemCount(k)
					if count > 0 then
						curritem = k
						currcount = count
						currvalue = v
					end
				end
			end
		end
		if curritem then
			PT3Bar:UpdateButton(button,curritem,currcount)
		else
			if locked then
				button:Hide()
			end
			button.icon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		end
	end
end

function PT3Bar:SetTooltip(frame)
	if frame.item and self.db.profile.tooltip then
		GameTooltip_SetDefaultAnchor(GameTooltip, frame)
		GameTooltip:SetHyperlink("item:"..frame.item)
	end
end

function PT3Bar:SetButtonHidden(button, v)
	local moveoverlay = button.moveoverlay
	if v then
		if not moveoverlay.hiddentex then
			moveoverlay.hiddentex = moveoverlay:CreateTexture(nil, 'BACKGROUND')
			moveoverlay.hiddentex:SetAllPoints(moveoverlay)
		end
		moveoverlay.hiddentex:SetTexture(0, 1, 0, 0.6)
	else
		if moveoverlay.hiddentex then
			moveoverlay.hiddentex:SetTexture(0, 0, 0, 0)
		end
	end
end

function PT3Bar:UpdateButton(frame,itemid,count)
	frame:SetAttribute("item", getname(itemid))
	if frame.hidden then
		if locked then
			frame:Hide()
		else
			frame:Show()
		end
	else
		frame:Show()
	end
	frame:SetAlpha(1)
	frame.item = itemid
	frame.icon:SetTexture(select(10,GetItemInfo(itemid)))
	frame.count:SetText(count)
end

function PT3Bar:UpdateCooldowns()
	for _,frame in pairs(buttons) do
		self:UpdateCooldown(frame)
	end
end

function PT3Bar:UpdateCooldown(frame)
	if frame.item and not frame.hidden then
		CooldownFrame_SetTimer(frame.cooldown,GetItemCooldown(frame.item))
	end
end